import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.EmptyBorder;
import java.io.*;
//must have jxl folder in main budget tool folder
import jxl.Workbook;
import jxl.*;
import jxl.write.*;
import java.util.LinkedList;

/* Print class collects data from Review, Edit and Budgetele classes to compile a comprehensive 
 * report in excel for ease of printing and pdf conversion.
 */
class Print extends JPanel implements ActionListener{
    //required project serialVersionUID
	private static final long serialVersionUID = -1154451023773719212L;
    private File f;                 //read files
    private File file;              //book file
    private globalvar g;            //global variable extraction
    private LinkedList<String> q;   //String type linked list for reading elements
    
    private JFileChooser fc;        //filechooser item
    private XlsFilter xls = new XlsFilter();    //create filter for xls printing
    
    private JLabel infotext = new JLabel();     //Instruction label for header
    
    private JPanel info = new JPanel();         //Instruction panel
    private JPanel buttons = new JPanel();      //Button panel
    
    JButton save = new JButton("Export to .xls");   //Button
    
    private double tax = 1.13;                      //tax based on global stored value
                                                    //see SetDefaults for editting
    
    /* Approximately 190 actual materials, this size will allow for an easy addition 
     * of 10 more materials. Individual values separated into 5 arrays for ease of use.
     */
    private String mat[] = new String[200];
    private String matunit[] = new String[200];
    private double matcost[] = new double[200];
    private double moolelo[] = new double[200];
    private double matco2[] = new double[200];
    
    private jxl.write.Label l;                      //xls text cell
    
    /* Constructor method created in initial run of budget.jar, will be created 
     * immediately as JTabbedPane tab.
     */
	public Print(){
        setLayout(new GridLayout(3,1));             //basic layout
		save.addActionListener(this);               //add button action
		
        //edit info Text
		infotext.setText("<HTML><P>This page will export the budget to excel format." +
				"<P>Please ensure that when typing the name of the file to export, that" +
				"<P>\".xls\" is included as the suffix.</HTML>");
		info.add(infotext);
        
		// add buttons to buttons panel
		buttons.setBorder(new EmptyBorder(5,70,5,70));
		buttons.add(save);
        
		// add panels to main panel
		add(info);
		add(buttons);
    }
    /* Update global variable and stored global items. Method is called by globalvar 
     * when file is created or opened in either Welcome or ShowInfo pages.
     */
    public void updateGlobe(globalvar globe){   //update all needed globally stored values into Print Frame
		g = globe;
        tax = g.getTax();
        mat = g.updateList();
        matunit = g.updateUnit();
        matcost = g.updateCost();
        moolelo = g.updateMo();
        matco2 = g.updateCO2();
	}
    /* Globalvar object will send current working file, sent from Welcome or ShowInfo pages.
     */
	public void updateFile(File f2){
		f = f2;
	}
    /*handles creation of a fileChooser upon clicking button as well as running read and save functions.
     */
    public void actionPerformed(ActionEvent e){ 
        /* create file chooser, not cleaned for the purpose of easy reprinting */
        if(fc == null){
            fc = new JFileChooser();
            fc.addChoosableFileFilter(xls);
            fc.setFileFilter(xls);
            fc.setAcceptAllFileFilterUsed(false);
        }
        int max = 0;
        if(e.getSource() == save){                              //save button action
            int returnVal = fc.showSaveDialog(Print.this);      
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                file = fc.getSelectedFile();
                //save file here
                if(xls.acceptFile(file)){
                    max = scanElements();                  //scan elements in file
                    try                                    //try opening a workbook
                    {
                        WorkbookSettings ws = new WorkbookSettings();
                        //ws.setLocale(new Locale("en", "EN")); //may need later
                        WritableWorkbook workbook = Workbook.createWorkbook(file, ws);
                        WritableSheet s;
                        //run through queue and operate on each item
                        for(int i = 0; i< max; i++){            
                            System.out.println(q.peek());
                            s = workbook.createSheet((String)q.peek(), i);
                            readFile((String)q.poll(), s);
                            writeDataSheet(s);
                        }
                        workbook.write();
                        workbook.close();
                    }
                    catch (IOException ioe)
                    {
                        ioe.printStackTrace();
                    }
                    catch (WriteException we)
                    {
                        we.printStackTrace();
                    }
                }else
                    file = null;
            }
        }
        max = 0;
    }
    /*
     * Read the original file as Element.java does to find all
     * sheets to write based on elements. Put said elements into
     * a linked list and treat it as a queue.
     */
    public int scanElements(){
        q = new LinkedList<String>();
        int max = 0;
        int count = 0;
        try{
            BufferedReader input =  new BufferedReader(new FileReader(f));
            try{
                String line = null;
                while ((line = input.readLine()) != null){
                    if(count>=9 && (line.equals(""))){
                        count--;
                        continue;
                    }
                    if(line.equals("")){
                        continue;
                    }
                    if(count>=9){
                        q.offer(line);
                        max++;
                    }
                    count++;
                }
            }finally{
                input.close();
            }
        }catch (IOException ioe){
            ioe.printStackTrace();
        }
        return max;
    }
    /*Write majority of label data to excel sheet, this function is more of an 
     * initialization, writing all static cells only.
     */
    private void writeDataSheet(WritableSheet s) 
    throws WriteException
    {
        
        /* Format the Font */
        
        WritableFont wf = new WritableFont(WritableFont.ARIAL, 10, WritableFont.BOLD);
        WritableCellFormat cf = new WritableCellFormat(wf);
        //WritableCellFormat ff = new WritableCellFormat(NumberFormats.FLOAT); 
        cf.setWrap(true);cf.setShrinkToFit(true);
        //ff.setWrap(true);ff.setShrinkToFit(true);
        
        /* Creates Label and writes date to one cell of sheet, no easy way to do this. 
         * Simply write a new cell then add it to the sheet. Everything is manual.
         */
        
        l = new jxl.write.Label(0,0,"Project",cf);
        s.addCell(l);
        l = new jxl.write.Label(1,0,"LABOUR",cf);
        s.addCell(l);
        l = new jxl.write.Label(2,0,"Hours",cf);
        s.addCell(l);
        l = new jxl.write.Label(3,0,"Rate",cf);
        s.addCell(l);
        l = new jxl.write.Label(4,0,"Totals",cf);
        s.addCell(l);
        l = new jxl.write.Label(5,0,"Production:",cf);
        s.addCell(l);
        l = new jxl.write.Label(1,1,"Set Labour:",cf);
        s.addCell(l);
        l = new jxl.write.Label(5,1,"Theatre:",cf);
        s.addCell(l);
        l = new jxl.write.Label(0,2,"Stage Floor",cf);
        s.addCell(l);
        l = new jxl.write.Label(1,2,"Paint Labour:",cf);
        s.addCell(l);
        l = new jxl.write.Label(5,2,"Version:",cf);
        s.addCell(l);
        l = new jxl.write.Label(1,3,"Set Materials:",cf);
        s.addCell(l);
        l = new jxl.write.Label(5,3,"Date:",cf);
        s.addCell(l);
        l = new jxl.write.Label(3,4,"Project Total:",cf);
        s.addCell(l);
        l = new jxl.write.Label(0,5,"Material",cf);
        s.addCell(l);
        l = new jxl.write.Label(1,5,"Unit",cf);
        s.addCell(l);
        l = new jxl.write.Label(2,5,"Cost Per",cf);
        s.addCell(l);
        l = new jxl.write.Label(3,5,"Quantity",cf);
        s.addCell(l);
        l = new jxl.write.Label(4,5,"Cost & Tax",cf);
        s.addCell(l);
        l = new jxl.write.Label(5,5,"CO2ekg",cf);
        s.addCell(l);
        l = new jxl.write.Label(6,5,"Green Score",cf);
        s.addCell(l);
        l = new jxl.write.Label(7,5,"Result CO2ekg",cf);
        s.addCell(l);
    }
    /*Similar to other classes we read element data stored previously. Write number information 
     * into the same excel sheet as before. This method handles all variable cells, with 
     * information gathered from elements created previously.
     */
    public void readFile(String filename, WritableSheet s)throws WriteException{
        jxl.write.Number n[] = new jxl.write.Number[100];       //array of Number type cells
        jxl.write.Number nco2[] = new jxl.write.Number[50];     //array of co2 values
        jxl.write.Number nmo[] = new jxl.write.Number[50];      //array of mo'olelo values
        jxl.write.Number ntot[] = new jxl.write.Number[50];     //array of total co2 values
        jxl.write.Number totaln[] = new jxl.write.Number[50];   //created to calculate total sums
        
        JComboBox jcb = new JComboBox(mat);
        String fname;
        WritableFont wf = new WritableFont(WritableFont.ARIAL, 10);
        WritableCellFormat cf = new WritableCellFormat(wf);
        WritableCellFormat ff = new WritableCellFormat(NumberFormats.FLOAT); 
        cf.setWrap(true);cf.setShrinkToFit(true);
        ff.setWrap(true);ff.setShrinkToFit(true);
        
        double num = 0;
        int numCount = 0;
        int numTotal = 5;   //starts at 5 because of the first 4 summary totals
        int num2 = 0;       //counts co2 info index
        int index = 0;
        /*try opening and reading an element file. Formatted to put information into 
         * given arrays to be used to other calculations done in this method.
         */
        try{
            fname = g.getDirectory() + "/Element/" + filename + ".txt";
            f = new File(fname);
            if(f!=null){
                BufferedReader input =  new BufferedReader(new FileReader(f));
                try{
                    String delim = "[\t]";
                    String line = null; //not declared within while loop
                    int count = 0;
                    numCount = 0;
                    while ((line = input.readLine()) != null){
                        if(line.compareTo("-")!=0){
                            String[] token = line.split(delim);
                            if(count<5 && token[0]!=null && token[0].compareTo("-")!=0 && token[0].compareTo("null")!=0){
                                System.out.println(token[0]);
                                if(token[0].compareTo("null")==0)
                                	num = 0;
                                else
                                	num = Double.parseDouble(token[0]);
                                n[numCount++] = new jxl.write.Number(count+2,1,num,ff);
                            }else if(token[0].compareTo("-")==0){
                                count++;
                                continue;
                            }if(count<5 && token[1]!=null && token[1].compareTo("null")!=0){
                            	if(token[0].compareTo("null")==0)
                                	num = 0;
                            	else
                            		num = Double.parseDouble(token[1]);
                                n[numCount++] = new jxl.write.Number(count+2,2,num,ff);
                            }if(count>=5 && token[0]!=null && token[0].compareTo("-")!=0){
                                l = new jxl.write.Label(0,count+1,token[0],cf);
                                s.addCell(l);
                                jcb.setSelectedItem(token[0]);
                                index = jcb.getSelectedIndex();
                                if(token[1].compareTo("-")!=0){
                                    num = Double.parseDouble(token[1]);
                                    n[numCount++] = new jxl.write.Number(2,count+1,num,ff);
                                }
                                if(token[2].compareTo("-")!=0){
                                    num = Double.parseDouble(token[2]);
                                    n[numCount++] = new jxl.write.Number(3,count+1,num,ff);
                                }else{
                                    n[numCount++] = new jxl.write.Number(3,count+1,0,ff);
                                }
                                l = new jxl.write.Label(1,count+1,matunit[index],cf);
                                s.addCell(l);
                                totaln[numTotal++] = new jxl.write.Number(4,count+1,tax*n[numCount-1].getValue()*n[numCount-2].getValue(),ff);
                                nco2[num2] = new jxl.write.Number(5,count+1,matco2[index]*n[numCount-3].getValue(),ff);
                                nmo[num2] = new jxl.write.Number(6,count+1,moolelo[index],ff);
                                if(moolelo[index] == 4)
                                    num = 0.6;
                                else if (moolelo[index] == 3)
                                    num = 0.7;
                                else if (moolelo[index] == 2)
                                    num = 0.8;
                                else if (moolelo[index] == 1)
                                    num = 0.9;
                                else
                                    num = 1.0;
                                ntot[num2++]= new jxl.write.Number(7,count+1,nco2[num2-1].getValue()*num,ff);
                            }
                        }
                        count++;
                    }
                    //begin calculating sums for totals row
                    num = 0;
                    for(int i = 4; i<numTotal;i++){
                        if(totaln[i]!=null)
                            num += totaln[i].getValue();
                    }
                    l = new jxl.write.Label(3,count+3,"Material Total:",cf);
                    s.addCell(l);
                    totaln[numTotal] = new jxl.write.Number(4,count+3,num,ff);
                    if(n[0]!=null && n[1]!=null)
                    	totaln[0] = new jxl.write.Number(4,1,n[1].getValue()*n[0].getValue(),ff);
                    if(n[2]!=null && n[3]!=null)
                    	totaln[1] = new jxl.write.Number(4,2,n[2].getValue()*n[3].getValue(),ff);
                    if(totaln[numTotal]!=null)
                    	totaln[2] = new jxl.write.Number(4,3,totaln[numTotal].getValue(),ff);
                    if(totaln[0]!=null && totaln[2]!=null)
                    	totaln[3] = new jxl.write.Number(4,4,totaln[0].getValue()+totaln[1].getValue()+totaln[2].getValue(),ff);
                    
                    for(int i = 0; i < numCount; i++){
                        if(n[i]!=null)
                            s.addCell(n[i]);
                        if(totaln[i]!=null && i<50)
                            s.addCell(totaln[i]);
                    }
                    
                    num = 0;
                    double co2 = 0;
                    double mo = 0;
                    for(int i = 0; i<num2; i++){
                        if(ntot[i]!=null)
                            num += ntot[i].getValue();
                        if(nco2[i]!=null)
                            co2 += nco2[i].getValue();
                        if(nmo[i]!=null){
                            mo += nmo[i].getValue();
                        }
                        s.addCell(nco2[i]);
                        s.addCell(nmo[i]);
                        s.addCell(ntot[i]);
                    }
                    nco2[num2] = new jxl.write.Number(5,count+3, co2, ff);
                    //ensure not dividing by 0 for empty element for mo'olelo avg
                    if(num2!=0)
                    	nmo[num2] = new jxl.write.Number(6,count+3,mo/num2,ff);
                    else
                    	nmo[num2] = new jxl.write.Number(6,count+3,0,ff);
                    ntot[num2] = new jxl.write.Number(7,count+3,num,ff);
                    s.addCell(nco2[num2]);
                    s.addCell(nmo[num2]);
                    s.addCell(ntot[num2]);
                }finally{
                    input.close();
                }
            }
        }catch (IOException ioe){
            System.out.println("read for "+filename+" did not work");
        }
        
        // read show info for ownership information to print.
        try{
            fname = g.getDirectory() + "/" + g.getFilename();
            f = new File(fname);
            if(f != null && f.exists()){
                BufferedReader input =  new BufferedReader(new FileReader(f));
                try {
                    String line = null; //not declared within while loop
                    int count = 0;
                    while ((line = input.readLine()) != null){
                        if(count == 0){
                            l = new jxl.write.Label(6,2,line,cf);
                        }
                        else if(count == 1){
                            l = new jxl.write.Label(6,3,line,cf);
                        }
                        else if(count == 2){
                            l = new jxl.write.Label(6,0,line,cf);
                        }
                        else if(count == 3){
                            l = new jxl.write.Label(6,1,line,cf);
                        }
                        else
                            break;
                        s.addCell(l);
                        count++;
                    }
                }finally{
                    input.close();
                }
            }
        }catch (IOException ioe){
            ioe.printStackTrace();
        }
    }
    public void reviewSheet(){
    	
    }
}

